/* ====================================================================
 * Copyright (c) 1996-2000 The Apache Software Foundation.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Software Foundation
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Software Foundation" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Software Foundation.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Software Foundation
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE Apache Software Foundation ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE Apache Software Foundation OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Software Foundation and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */

#ifndef AP_BUF_H
#define AP_BUF_H

#ifdef HAVE_SYS_UIO_H
#include <sys/uio.h>	/* for struct iovec */
#endif

typedef struct ap_bufel ap_bufel;
struct ap_bufel {
    ap_bufel *next;
    char *start;			/* first byte */
    char *end;				/* last byte + 1 */
    void (*free)(ap_bufel *e);		/* never NULL */
    void *data;				/* for use by free() */
};

typedef struct ap_buf ap_buf;
struct ap_buf {
    ap_bufel *head;
    ap_bufel **tail;
    unsigned size;
};

/* allocate a bufel */
API_EXPORT(ap_bufel *) ap_bufel_new(void);

/* destroy a bufel */
API_EXPORT(void) ap_bufel_destroy(ap_bufel *);

/* initialize a buf */
API_EXPORT(void) ap_buf_init(ap_buf *);

/* append a bufel to buf */
API_EXPORT(void) ap_buf_append(ap_buf *, ap_bufel *);

/* consume nbytes from beginning of buf -- call ap_bufel_destroy as
    appropriate, and/or modify start on last element */
API_EXPORT(void) ap_buf_consume(ap_buf *, int nbytes);

/* create an iovec of the elements in buf... return number of elements used */
API_EXPORT(int) ap_buf_to_iovec(ap_buf *, struct iovec *vec, int nvec);

/* catenate buf b onto buf a, buf b is empty after this */
API_EXPORT(void) ap_buf_catenate(ap_buf *a, ap_buf *b);

#endif
