/* ====================================================================
 * Copyright (c) 2000 The Apache Software Foundation.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Software Foundation
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Software Foundation" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Software Foundation.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Software Foundation
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE Apache Software Foundation ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE Apache Software Foundation OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.
 * For more information on the Apache Software Foundation and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "apr_mmap.h"
#include "apr_errno.h"
#include "apr_general.h"
#include "apr_lib.h"
#include "apr_file_io.h"

/* hmmm, what is a truly portable define for the max path
 * length on a platform?
 */
#define PATH_LEN 255

int main()
{
    ap_context_t *context;
    ap_mmap_t *themmap = NULL;
    ap_file_t *thefile = NULL;
    ap_finfo_t finfo;
    ap_int32_t flag = APR_READ;
    char *file1;
    
    fprintf (stdout,"APR MMAP Test\n*************\n\n");
    
    fprintf(stdout,"Creating context....................");    
    if (ap_create_context(&context, NULL) != APR_SUCCESS) {
        fprintf(stderr, "Failed.\n");
        exit(-1);
    }
    fprintf(stdout,"OK\n");
    
    file1 = (char*) ap_palloc(context, sizeof(char) * PATH_LEN);
    getcwd(file1, PATH_LEN);
    strncat(file1,"/testmmap.c",11);  

    fprintf(stdout, "Opening file........................");
    if (ap_open(&thefile, file1, flag, APR_UREAD | APR_GREAD, context) != APR_SUCCESS) {
        perror("Didn't open file");
        exit(-1);
    }
    else {
        fprintf(stdout, "OK\n");
    }
    
    fprintf(stderr, "Getting file size...................");
    if (ap_getfileinfo(&finfo, thefile) != APR_SUCCESS) {
        perror("Didn't get file information!");
        exit(-1);
    }
    else {
        fprintf(stdout, "%d bytes\n", finfo.size);
    }  
    
    fprintf(stdout,"Trying to mmap the file..............");
    if (ap_mmap_create(&themmap, thefile, 0, finfo.size, context) != APR_SUCCESS) {
        fprintf(stderr,"Failed!\n");
        exit(-1);
    }
    fprintf(stdout,"OK\n");

    fprintf(stdout,"Trying to delete the mmap file......");
    if (ap_mmap_delete(themmap) != APR_SUCCESS) {
        fprintf(stderr,"Failed!\n");
        exit (-1);
    }
    fprintf(stdout,"OK\n");
    
    
    fprintf (stdout,"\nTest Complete\n");
    
    return 1;
}
