/* ====================================================================
 * Copyright (c) 1995-2000 The Apache Software Foundation.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.  
 * 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 * 
 * 3. All advertising materials mentioning features or use of this 
 *    software must display the following acknowledgment: 
 *    "This product includes software developed by the Apache Software Foundation 
 *    for use in the Apache HTTP server project (http://www.apache.org/)." 
 * 
 * 4. The names "Apache Server" and "Apache Software Foundation" must not be used to 
 *    endorse or promote products derived from this software without 
 *    prior written permission. For written permission, please contact 
 *    apache@apache.org. 
 * 
 * 5. Products derived from this software may not be called "Apache" 
 *    nor may "Apache" appear in their names without prior written 
 *    permission of the Apache Software Foundation. 
 * 
 * 6. Redistributions of any form whatsoever must retain the following 
 *    acknowledgment: 
 *    "This product includes software developed by the Apache Software Foundation 
 *    for use in the Apache HTTP server project (http://www.apache.org/)." 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE Apache Software Foundation ``AS IS'' AND ANY 
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE Apache Software Foundation OR 
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE. 
 * ==================================================================== 
 * 
 * This software consists of voluntary contributions made by many 
 * individuals on behalf of the Apache Software Foundation and was originally based 
 * on public domain software written at the National Center for 
 * Supercomputing Applications, University of Illinois, Urbana-Champaign. 
 * For more information on the Apache Software Foundation and the Apache HTTP server 
 * project, please see <http://www.apache.org/>. 
 * 
 */ 

#include <stdio.h> 
#include <kernel/OS.h>
#include <math.h>

#include "mutex.h"
#include "httpd.h" 
#include "acceptlock.h"
#include "ap_mpm.h"
#include "mpmt_beos.h"

/* Number of cross-process locks we're managing */
static int lock_count;
static be_mutex_t *accept_mutexes = NULL;
static int *have_accept_mutex;
area_id	mutex_area;
void * area_ptr;

char *accept_mutex; /* pointer to area */
/* Number of intra-process locks we're managing */
static int intra_lock_count;

/* Intraprocess locking used by other serialization techniques */
static be_mutex_t *intra_mutex = NULL;

static ap_status_t intra_mutex_cleanup(void *foo)
{
    int i;

    for (i = 0; i < intra_lock_count; i++) {
        (void) be_mutex_destroy(&intra_mutex[i]);
    }
    return APR_SUCCESS;
}

void intra_mutex_init(ap_context_t *p, int number_of_locks)
{
    int i;
    intra_lock_count = number_of_locks;
    intra_mutex = (be_mutex_t *)ap_palloc(p, intra_lock_count * sizeof(be_mutex_t));
    for (i = 0; i < intra_lock_count; i++) {       
        if (be_mutex_init(&intra_mutex[i], NULL) == -1) {
            perror("intra_mutex_init");
	        clean_child_exit(APEXIT_CHILDFATAL);
        }
    }
    ap_register_cleanup(p, NULL, intra_mutex_cleanup, ap_null_cleanup);
}

void intra_mutex_on(int locknum)
{
    be_mutex_lock(&intra_mutex[locknum]);
}

void intra_mutex_off(int locknum)
{
    be_mutex_unlock(&intra_mutex[locknum]);
}


ap_status_t accept_mutex_child_cleanup(void *foo)
{
    int i;

    if (accept_mutex != NULL) {
        for (i = 0; i < lock_count; i++) {
	    	if (have_accept_mutex[i])
  	        	be_mutex_unlock(&accept_mutexes[i]);
		}
    }
    return APR_SUCCESS;
}

void accept_mutex_child_init(ap_context_t *p)
{
    ap_register_cleanup(p, NULL, accept_mutex_child_cleanup, ap_null_cleanup);
}

ap_status_t accept_mutex_cleanup(void *foo)
{ 
	delete_area(mutex_area);
    accept_mutex = NULL;
    return APR_SUCCESS;
}

void accept_mutex_init(ap_context_t *p, int number_of_locks)
{
    int i;
    float size = number_of_locks * sizeof(be_mutex_t);
    
    size_t	nPages = ceil(size / B_PAGE_SIZE);
    if (nPages == 0) nPages = 1;
    area_ptr = NULL;
    mutex_area = create_area("httpd_accept_mutex", &area_ptr, B_ANY_ADDRESS,
	B_PAGE_SIZE * nPages, B_FULL_LOCK, B_READ_AREA|B_WRITE_AREA);
	   
    lock_count = number_of_locks;
    have_accept_mutex = (int *)ap_palloc(p, lock_count * sizeof(int));

    if (area_ptr == NULL) {
        perror("create_area");
        exit(APEXIT_INIT);
    }
    accept_mutexes = (be_mutex_t *)area_ptr;
    for (i = 0; i < lock_count; i++) {
		if ((errno = be_mutex_init(&accept_mutexes[i], NULL))) {
		    perror("be_mutex_init");
	    	exit(APEXIT_INIT);
		}
    }
    ap_register_cleanup(p, NULL, accept_mutex_cleanup, ap_null_cleanup);
}

void accept_mutex_on(int locknum)
{
    int err;
    if ((err = be_mutex_lock(&accept_mutexes[locknum]))) {
	    errno = err;
	    perror("be_mutex_lock");
	    clean_child_exit(APEXIT_CHILDFATAL);
    }
    have_accept_mutex[locknum] = 1;
}

void accept_mutex_off(int locknum)
{
    int err;
    if ((err = be_mutex_unlock(&accept_mutexes[locknum]))) {
	    errno = err;
	    perror("be_mutex_lock");
	    clean_child_exit(APEXIT_CHILDFATAL);
    }
    have_accept_mutex[locknum] = 0;
}
