/* ====================================================================
 * Copyright (c) 1995-2000 The Apache Software Foundation.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Software Foundation
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Software Foundation" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Software Foundation.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Software Foundation
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE Apache Software Foundation ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE Apache Software Foundation OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Software Foundation and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */

/* main_win32.c - Apache executable stub file for Win32
 * This file's purpose in life is to load, and call the
 * "real" main function, apache_main(), located in ApacheCore.dll
 *
 * This was done because having the main() function in a DLL,
 * although Win32 allows it, seemed wrong. Also, MSVC++ won't
 * link an executable without at least one object file. This
 * satistifies that requirement.
 */
#define CORE_PRIVATE
#include "httpd.h"
#include "http_main.h"
#include "http_config.h"
#include "apr_getopt.h"
#include "service.h"
#include "http_log.h"
#include "registry.h"
#include "ap_config.h"

__declspec(dllexport) int apache_main(int argc, char *argv[]);
#define DEFAULTSERVICENAME "Apache"

static const char* read_config_cmd(char *filename, char *cmd, ap_context_t *p)
{
    ap_status_t rc;
    char l[MAX_STRING_LEN];
    const char *args, *cmd_name;
    const char *s = NULL;
    configfile_t *config_file = NULL;

    rc = ap_pcfg_openfile(&config_file, p, filename);
    if (rc != APR_SUCCESS) {
        printf("Apache: Unable to open %s.\n", filename);
        return NULL;
    }
    while (!(ap_cfg_getline(l, MAX_STRING_LEN, config_file))) {
        if ((l[0] == '#') || (!l[0]))
            continue;
        args = l;
        cmd_name = ap_getword_conf(p, &args);
        if (!stricmp(cmd_name,cmd)) {
            s = args;
            break;
        }
    }
    ap_cfg_closefile(config_file);
    if (s == NULL) {
        printf("Apache: Did not find directive %s in %s.\n", cmd, filename);
    }
    return s;
}
static void signal_parent(const char* file, const char* signal, 
                          ap_context_t *p) 
{
    HANDLE event;
    char prefix[20];
    char *EventName;
    FILE *fp;
    int nread;
    int end;
    fp = fopen(file, "r");
    if (!fp) {
	printf("Cannot read apache PID file %s. Error = %d\n", file, errno);
        return;
    }
    prefix[0] = 'a';
    prefix[1] = 'p';

    nread = fread(prefix+2, 1, sizeof(prefix)-3, fp);
    if (nread == 0) {
	fclose(fp);
	printf("PID file %s was empty\n", file);
        return;
    }
    fclose(fp);

    /* Terminate the prefix string */
    end = 2 + nread - 1;
    while (end > 0 && (prefix[end] == '\r' || prefix[end] == '\n'))
	end--;
    prefix[end + 1] = '\0';

    /* Build the event name. Should be one of the following...
     * apPID_shutdown
     * apPID_restart
     */
    EventName = ap_pstrcat(p,prefix,"_",signal,NULL);
    printf("event name = %s\n", EventName);
    event = OpenEvent(EVENT_ALL_ACCESS, FALSE, EventName);
    printf("event handle = %d\n", event);
    if (event == NULL) {
	printf("Unable to open event %s.\n", EventName);
        return;
    }
    SetEvent(event);
    ResetEvent(event);
    CloseHandle(event);
    return;
    
}
int main(int argc, char *argv[])
{
    ap_context_t *pwincmd;
    int c;
    char *ch;
    char *service_name = NULL; //DEFAULTSERVICENAME;
    char *signal = NULL;
    char *server_root = NULL;
    char *server_confname = SERVER_CONFIG_FILE;
    char *temp_server_confname = NULL;
    char cwd[MAX_STRING_LEN];
    char **new;

    BOOLEAN install = FALSE;
    BOOLEAN uninstall = FALSE;

    ap_array_header_t *cmdtbl;
    int new_argc = 0;

    ap_create_context(&pwincmd, NULL);
    if (pwincmd == NULL) {
        exit(0);
    }

    /* Set up default server_root */
    if (!GetCurrentDirectory(sizeof(cwd),cwd)) {
        ap_log_error(APLOG_MARK,APLOG_ERR, GetLastError(), NULL, "GetCurrentDirectory() failure");
        exit(0);
    }
    server_root = ap_os_canonical_filename(pwincmd, cwd);

    cmdtbl = ap_make_array(pwincmd, 1, sizeof(char *));

    /* Add the new_argv[0] */
    new = (char **) ap_push_array(cmdtbl);
    *new = argv[0];
    new_argc++;

    /* Service application
     * Configuration file in registry at:
     * HKLM\System\CurrentControlSet\Services\[Svc name]\Parameters\ConfPath
     */
    if (isProcessService()) {
        service_main(apache_main, argc, argv);
        exit(0);
    }

    /* Munch away at the command line arguments... */
    while (ap_getopt(pwincmd, argc, argv, "n:k:iuC:c:d:f:vVlLth?", &c) == APR_SUCCESS) {
        switch (c) {
        case 'k':
            signal = ap_optarg;
            break;
        case 'i':
            install = TRUE;
            break;
        case 'u':
            uninstall = TRUE;
            break;
        case 'n':
            service_name = ap_pstrdup(pwincmd, ap_optarg);
            if (isValidService(ap_optarg)) {
                ap_registry_get_service_conf(pwincmd, &server_confname, ap_optarg);
            }
            break;
        case 'f':
            server_confname = ap_optarg; 
            break;    
        case 'd':
            server_root = ap_os_canonical_filename(pwincmd, ap_optarg);
            break;
        default:
            new = (char **) ap_push_array(cmdtbl);
            ch = ap_pstrndup(pwincmd, (char*)&c, 1);
            *new = ap_pstrcat(pwincmd,"-", ch, NULL);
            new_argc++;
            if (ap_optarg) {
                new = (char **) ap_push_array(cmdtbl);
                *new = ap_optarg;
                new_argc++;
            }
            break;
        }
    }

    /* Determine server_confname. 
     * server_confname is found in this order:
     * (1) -f or -n
     * (2) [-d]/SERVER_CONFIG_FILE
     * (3) ./SERVER_CONFIG_FILE
     * (4) [Registry: HKLM\Software\[product]\ServerRoot]/SERVER_CONFIG_FILE
     * (5) /HTTPD_ROOT/SERVER_CONFIG_FILE (Humm, I don't think we should even try this... )
     */
    temp_server_confname = ap_pstrdup(pwincmd, server_confname);
    if (!ap_os_is_path_absolute(temp_server_confname)) {
        temp_server_confname = ap_make_full_path(pwincmd, server_root, temp_server_confname);
    }
    temp_server_confname = ap_os_canonical_filename(pwincmd, temp_server_confname);
    ap_getparents(temp_server_confname);
    ap_no2slash(temp_server_confname);

    if (access(temp_server_confname, 0)) {
        ap_registry_get_server_root(pwincmd, &server_root);
        if (!server_root) {
            server_root = HTTPD_ROOT;
        }
        server_root = ap_os_canonical_filename(pwincmd, server_root);

        temp_server_confname = ap_pstrdup(pwincmd, server_confname);
        if (!ap_os_is_path_absolute(temp_server_confname)) {
            temp_server_confname = ap_make_full_path(pwincmd, server_root, temp_server_confname);
        }
        temp_server_confname = ap_os_canonical_filename(pwincmd, temp_server_confname);
        ap_getparents(temp_server_confname);
        ap_no2slash(temp_server_confname);
    }

    server_confname = temp_server_confname;

    /* By now, we should have canonicalized absolute paths to 
     * server_root and server_confname */

    /* Handle -i (install as a service) */
    if (install) {
        if (!service_name)
            service_name = DEFAULTSERVICENAME;
        InstallService(service_name, server_confname);
        exit(0);
    }
    /* Handle -u (uninstall service) */
    if (uninstall) {
        if (!service_name)
            service_name = DEFAULTSERVICENAME;
        RemoveService(service_name);
        exit(0);
    }

    /* Handle -k <signal> -n <service_name> */
    if (service_name && signal) {
        send_signal_to_service(service_name, signal);
        exit(0);
    }

    /* Handle -k restart|shutdown */
    if (signal && strcasecmp(signal, "start")) {
        char *s;
        s = read_config_cmd(server_confname, "pidfile", pwincmd);
        if (s == NULL) {
            exit(0);
        }
        if (!ap_os_is_path_absolute(s)) {
            s = ap_make_full_path(pwincmd, server_root, s);
        }
        signal_parent(s, signal, pwincmd);
        exit(0);
    }

    /* Complete building the new argv list. Need to add:
     * -f server_conf
     * -d server_root
     */
    new = (char **) ap_push_array(cmdtbl);
    *new = "-f";
    new = (char **) ap_push_array(cmdtbl);
    *new = server_confname;
    new = (char **) ap_push_array(cmdtbl);
    *new = "-d";
    new = (char **) ap_push_array(cmdtbl);
    *new = server_root;
    new_argc += 4;    

    /* Set optreset and optind to allow ap_getopt to work correctly
     * when called from http_main.c
     */
    ap_optreset = 1;
    ap_optind = 1;

    return apache_main(new_argc, (char**) cmdtbl->elts);
    
}
